import importlib


class Keeper:
    def __init__(self):
        pass

    def init(self, provider):
        if hasattr(self, provider):
            print(f'{provider} provider already set')
            return
        module = importlib.import_module(provider)
        module_class = getattr(module, 'Provider')
        setattr(self, provider, module_class())



"""
AWS Connection Manager
"""

import boto3

_CLIENTS = {}
_SESSIONS = {}


def get_client(profile, region, service):
    """
    Fetch an AWS Client, and store it for later use
    """
    client_key = (profile, region)
    session_key = profile

    if client_key not in _CLIENTS:
        if session_key not in _SESSIONS:
            _SESSIONS[session_key] = boto3.Session(profile_name=profile)
        _CLIENTS[client_key] = _SESSIONS[session_key].client(
            service, region_name=region)
    return _CLIENTS[client_key]



from aws import paginate
from aws.connection_manager import get_client


class Resource:
    def __init__(self, profiles, regions):
        self.profiles = profiles
        self.regions = regions

    @property
    def clients(self):
        return [
            get_client(profile=profile,
                       region=region,
                       service='cloudformation') for profile in self.profiles
            for region in self.regions
        ]

    def scrape(self):
        for client in self.clients:
            paginator = paginate.Paginator(client, 'describe_stacks')
            return paginator.scrape()



import importlib


class Provider:
    def __init__(self):
        self._profiles = None

    @property
    def profiles(self):
        return self._profiles

    @profiles.setter
    def profiles(self, profiles):
        self._profiles = profiles

    def scrape(self, resource):
        module = importlib.import_module(f'aws.{resource}')
        module_class = getattr(module, 'Resource')
        instance = module_class(profiles=self.profiles, regions=self.regions)
        return instance.scrape()



import boto3


class Paginator:
    def __init__(self, client, operation):
        self.client = client
        self.operation = operation

    def scrape(self):
        paginator = self.client.get_paginator(self.operation)
        response_iterator = paginator.paginate()
        return [response for response in response_iterator]
